"""Core nodes for Flowire workflow automation.

This package contains the built-in nodes for Flowire workflows.
Nodes are auto-discovered via entry points defined in pyproject.toml.
"""

__version__ = "0.1.0"
